#include <iostream>
#include <iomanip>
#include <string>
#include <algorithm>
#include <vector>  

#include "TCanvas.h"
#include "TGraph.h"
#include "TF1.h"
#include "TFile.h"
#include "TH1F.h"
#include "TH2F.h"
#include "TStyle.h"
#include "TROOT.h"
#include "TImage.h"
#include "TLatex.h"
#include "TLegend.h"
#include <TSystem.h>
#include <TMarker.h>
#include "TMath.h"
Double_t fbinw;
Double_t fgaus(Double_t *x, Double_t *par);
double nsim = 1e5;
void eff(int show=1000,int det=1000){
  gStyle->SetOptFit(0);
  gStyle->SetOptStat(0);
  gStyle->SetOptTitle(0);
  gStyle->SetTextFont(132);
  gStyle->SetLegendFont(132);
  
  gROOT->ForceStyle();
  const int n = 9;
  const int n1 = 4;
  double ene[n];
  double eff[3][2][n];
  TH1F* h;
  TFile *f;
  TF1* fu = new TF1("gaus",fgaus,100,200,3);
  fu->SetParameters(100000,1000,10);

  for(int i=0;i<n;i++){
    if(i<n1)
      ene[i] = 100*i+200;
    else
      ene[i] = 1000*(i-n1)+1000;
    f = new TFile(Form("histold/source%dkeV.root",(int)ene[i]));
    h = (TH1F*)f->Get("egam");
    fbinw = h->GetBinWidth(1);
    fu->SetRange(ene[i]-50,ene[i]+50);
    fu->SetParameters(h->Integral(ene[i]-100,ene[i]+100),ene[i],1);
    fu->SetParLimits(0,1,1e6);
    fu->SetParLimits(1,ene[i]-10,ene[i]+10);
    fu->SetParLimits(2,0.01,100);
    h->Fit(fu,"Rq");
    eff[0][0][i] = fu->GetParameter(0)/nsim*100;
    if(det==0&&i==show)
      return;
    h = (TH1F*)f->Get("MBegam");
    fbinw = h->GetBinWidth(1);
    h->Fit(fu,"Rq");
    eff[1][0][i] = fu->GetParameter(0)/nsim*100;
    if(det==1&&i==show)
      return;
    h = (TH1F*)f->Get("GRegam");
    fbinw = h->GetBinWidth(1);
    h->Fit(fu,"Rq");
    eff[2][0][i] = fu->GetParameter(0)/nsim*100;
    if(det==1&&i==show)
      return;
    h = (TH1F*)f->Get("egamAB");
    h->Fit(fu,"Rq");
    eff[0][1][i] = fu->GetParameter(0)/nsim*100;
    if(det==3&&i==show)
      return;
    h = (TH1F*)f->Get("MBegamAB");
    h->Fit(fu,"Rq");
    eff[1][1][i] = fu->GetParameter(0)/nsim*100;
    if(det==4&&i==show)
      return;
    h = (TH1F*)f->Get("GRegamAB");
    h->Fit(fu,"Rq");
    eff[2][1][i] = fu->GetParameter(0)/nsim*100;
    if(det==5&&i==show)
      return;
 
    cout << i << "\t"<< ene[i] << "\t" << eff[0][0][i] << "\t" << eff[1][0][i] << "\t" << eff[2][0][i] << endl;
  }
  TGraph* geff[3][2];
  for(int i=0;i<3;i++){
    geff[i][0] = new TGraph(n,ene,eff[i][0]);
    geff[i][1] = new TGraph(n,ene,eff[i][1]);
  } 
  TPad *p[2];
  TCanvas *c  = new TCanvas("c","c",800,300);
  c->Divide(2,1);
  char* title[2] =  {(char*)"singles",(char*)"add-back"};
  TLatex* tl[2];
  for(int i=0;i<2;i++){
    c->cd(1+i);
    p[i] = new TPad(Form("p%d",i),Form("p%d",i),0,0,1,1);
    p[i]->SetTopMargin(0.01);
    p[i]->SetLeftMargin(0.08);
    p[i]->SetBottomMargin(0.125);
    p[i]->SetRightMargin(0.01);
    p[i]->Draw();
    // p[i]->SetLogx();
    // p[i]->SetLogy();
    p[i]->cd();
    geff[0][i]->Draw("AP");
    geff[0][i]->SetMarkerStyle(20);
    geff[1][i]->SetMarkerStyle(21);
    geff[2][i]->SetMarkerStyle(34);
    geff[1][i]->SetMarkerColor(2);
    geff[2][i]->SetMarkerColor(3);
    geff[1][i]->Draw("P");
    geff[2][i]->Draw("P");
    geff[0][i]->GetYaxis()->SetTitle("#varepsilon (%)");
    geff[0][i]->GetYaxis()->SetTitleSize(0.065);
    geff[0][i]->GetYaxis()->SetTitleOffset(0.55);
    geff[0][i]->GetYaxis()->SetLabelSize(0.055);
    geff[0][i]->GetYaxis()->SetLabelOffset(0.01);
    geff[0][i]->GetXaxis()->SetTitle("E_{#gamma} (keV)");
    geff[0][i]->GetXaxis()->SetTitleSize(0.065);
    geff[0][i]->GetXaxis()->SetTitleOffset(0.9);
    geff[0][i]->GetXaxis()->SetLabelSize(0.055);
    geff[0][i]->GetXaxis()->SetLabelOffset(0.005);
    geff[0][i]->GetXaxis()->SetRangeUser(50,5700);
    geff[0][i]->GetYaxis()->SetRangeUser(0.,6.3);
    geff[0][i]->GetXaxis()->SetTitleFont(132);
    geff[0][i]->GetYaxis()->SetTitleFont(132);
    geff[0][i]->GetXaxis()->SetLabelFont(132);
    geff[0][i]->GetYaxis()->SetLabelFont(132);
    tl[i] = new TLatex(0.25,0.9,title[i]);
    tl[i]->SetNDC();
    tl[i]->SetTextSize(0.09);
    tl[i]->Draw();
  }
  TLegend *l = new TLegend(0.50, 0.69, 0.98, 0.97);
  l->AddEntry(geff[0][0], "HR array", "P");
  l->AddEntry(geff[1][0], "MB detectors", "P");
  l->AddEntry(geff[2][0], "tracking detectors", "P");
  l->SetTextSize(0.075);
  l->SetBorderSize(0);
  l->SetFillColor(0);
  p[0]->cd();
  l->Draw();
  c->SaveAs("sourceeff.eps");

}
void eff_newconfig(int show=1000,int det=1000){
  gStyle->SetOptFit(0);
  gStyle->SetOptStat(0);
  gStyle->SetOptTitle(0);
  gStyle->SetTextFont(132);
  gStyle->SetLegendFont(132);
  
  gROOT->ForceStyle();
  const int n = 9;
  const int n1 = 4;
  double ene[n];
  double eff[4][2][n];
  TH2F* h2;
  TH1F* h;
  TFile *f;
  TF1* fu = new TF1("gaus",fgaus,100,200,3);
  fu->SetParameters(100000,1000,10);

  for(int i=0;i<n;i++){
    if(i<n1)
      ene[i] = 100*i+200;
    else
      ene[i] = 1000*(i-n1)+1000;
    cout << ene[i] << endl;
    f = new TFile(Form("hist/source%dkeV.root",(int)ene[i]));
    //f->ls();
    h2 = (TH2F*)f->Get("egam_detector");
    h = (TH1F*)h2->ProjectionY("al",1,12);
    fbinw = h->GetBinWidth(1);
    fu->SetRange(ene[i]-50,ene[i]+50);
    fu->SetParameters(h->Integral(ene[i]-100,ene[i]+100),ene[i],1);
    fu->SetParLimits(0,1,1e6);
    fu->SetParLimits(1,ene[i]-10,ene[i]+10);
    fu->SetParLimits(2,0.01,100);
    h->Fit(fu,"Rq");
    eff[0][0][i] = fu->GetParameter(0)/nsim*100;
    if(det==0&&i==show)
      return;

    h = (TH1F*)h2->ProjectionY("mb",3,10);
    fbinw = h->GetBinWidth(1);
    h->Fit(fu,"Rq");
    eff[1][0][i] = fu->GetParameter(0)/nsim*100;
    if(det==1&&i==show)
      return;

    h = (TH1F*)h2->ProjectionY("cl",1,2);
    fbinw = h->GetBinWidth(1);
    h->Fit(fu,"Rq");
    eff[2][0][i] = fu->GetParameter(0)/nsim*100;
    if(det==2&&i==show)
      return;

    h = (TH1F*)h2->ProjectionY("sc",11,12);
    fbinw = h->GetBinWidth(1);
    h->Fit(fu,"Rq");
    eff[3][0][i] = fu->GetParameter(0)/nsim*100;
    if(det==3&&i==show)
      return;


    
    h2 = (TH2F*)f->Get("egamAB_detector");
    h = (TH1F*)h2->ProjectionY("al",1,12);
    fbinw = h->GetBinWidth(1);
    h->Fit(fu,"Rq");
    eff[0][1][i] = fu->GetParameter(0)/nsim*100;
    if(det==4&&i==show)
      return;

    h = (TH1F*)h2->ProjectionY("mb",3,10);
    fbinw = h->GetBinWidth(1);
    h->Fit(fu,"Rq");
    eff[1][1][i] = fu->GetParameter(0)/nsim*100;
    if(det==5&&i==show)
      return;

    h = (TH1F*)h2->ProjectionY("cl",1,2);
    h->Fit(fu,"Rq");
    eff[2][1][i] = fu->GetParameter(0)/nsim*100;
    if(det==6&&i==show)
      return;
    
    h = (TH1F*)h2->ProjectionY("sc",11,12);
    h->Fit(fu,"Rq");
    eff[3][1][i] = fu->GetParameter(0)/nsim*100;
    if(det==7&&i==show)
      return;

    cout << i << "\t"<< ene[i] << "\t" << eff[0][0][i] << "\t" << eff[1][0][i] << endl;
  }
  TGraph* geff[4][2];
  for(int i=0;i<4;i++){
    geff[i][0] = new TGraph(n,ene,eff[i][0]);
    geff[i][1] = new TGraph(n,ene,eff[i][1]);
  } 
  TPad *p[2];
  TCanvas *c  = new TCanvas("c","c",800,300);
  c->Divide(2,1);
  char* title[2] =  {(char*)"singles",(char*)"add-back"};
  TLatex* tl[2];
  for(int i=0;i<2;i++){
    c->cd(1+i);
    p[i] = new TPad(Form("p%d",i),Form("p%d",i),0,0,1,1);
    p[i]->SetTopMargin(0.01);
    p[i]->SetLeftMargin(0.1);
    p[i]->SetBottomMargin(0.125);
    p[i]->SetRightMargin(0.01);
    p[i]->Draw();
    // p[i]->SetLogx();
    // p[i]->SetLogy();
    p[i]->cd();
    geff[0][i]->Draw("AP");
    geff[0][i]->SetMarkerStyle(20);
    geff[1][i]->SetMarkerStyle(21);
    geff[1][i]->SetMarkerColor(2);
    geff[1][i]->Draw("P");
    geff[2][i]->SetMarkerStyle(22);
    geff[2][i]->SetMarkerColor(3);
    geff[2][i]->Draw("P");
    geff[3][i]->SetMarkerStyle(23);
    geff[3][i]->SetMarkerColor(4);
    geff[3][i]->Draw("P");
    geff[0][i]->GetYaxis()->SetTitle("#varepsilon (%)");
    geff[0][i]->GetYaxis()->SetTitleSize(0.065);
    geff[0][i]->GetYaxis()->SetTitleOffset(0.8);
    geff[0][i]->GetYaxis()->SetLabelSize(0.055);
    geff[0][i]->GetYaxis()->SetLabelOffset(0.01);
    geff[0][i]->GetXaxis()->SetTitle("E_{#gamma} (keV)");
    geff[0][i]->GetXaxis()->SetTitleSize(0.065);
    geff[0][i]->GetXaxis()->SetTitleOffset(0.9);
    geff[0][i]->GetXaxis()->SetLabelSize(0.055);
    geff[0][i]->GetXaxis()->SetLabelOffset(0.005);
    geff[0][i]->GetXaxis()->SetRangeUser(50,5700);
    geff[0][i]->GetYaxis()->SetRangeUser(0.,7.9);
    geff[0][i]->GetXaxis()->SetTitleFont(132);
    geff[0][i]->GetYaxis()->SetTitleFont(132);
    geff[0][i]->GetXaxis()->SetLabelFont(132);
    geff[0][i]->GetYaxis()->SetLabelFont(132);
    tl[i] = new TLatex(0.3,0.9,title[i]);
    tl[i]->SetNDC();
    tl[i]->SetTextSize(0.09);
    tl[i]->Draw();
  }
  TLegend *l = new TLegend(0.47, 0.62, 0.95, 0.97);
  l->AddEntry(geff[0][0], "HiCARI", "P");
  l->AddEntry(geff[1][0], "MB cluster", "P");
  l->AddEntry(geff[2][0], "Tracking detectors", "P");
  l->AddEntry(geff[3][0], "Super Clover", "P");
  l->SetTextSize(0.075);
  l->SetBorderSize(0);
  l->SetFillColor(0);
  p[0]->cd();
  l->Draw();
  c->SaveAs("sourceeff_newconfig.pdf");

}
Double_t fgaus(Double_t *x, Double_t *par){
  static Float_t sqrt2pi = TMath::Sqrt(2*TMath::Pi()), sqrt2 = TMath::Sqrt(2.);
  Double_t arg;
  Double_t norm  = par[0];
  Double_t mean  = par[1];
  Double_t sigma = par[2];
  if(sigma==0)
    return 0;
  arg = (x[0]-mean)/(sqrt2*sigma);
  return fbinw/(sqrt2pi*sigma) * norm * exp(-arg*arg);
}
